import { dateUtil } from '@csii/vx-util';

// 转化日期为年月日...
const changeDateToText = function(date) {
  const dateType = date.match(/[yMwdhms]/)[0];
  switch (dateType) {
    case 'M':
      return {
        text: '月',
        value: 'month',
      };
    case 'w':
      return {
        text: '周',
        value: 'week',
      };
    case 'd':
      return {
        text: '天',
        value: 'day',
      };
    case 'h':
      return {
        text: '小时',
        value: 'hour',
      };
    case 'm':
      return {
        text: '分钟',
        value: 'minute',
      };
    case 's':
      return {
        text: '秒',
        value: 'second',
      };
    case 'y':
    default:
      return {
        text: '年',
        value: 'year',
      };
  }
};

const yesterdayFun = (time) => {
  return dateUtil.changeDate('-1d', time, 'YYYY-MM-DD');
};

const dateSpanFun = (dateSpan, dateSpanFix, callback, value) => {
  if (!/^(-?\d+)([yMwdhms]s?)$/.test(dateSpan)) {
    console.error('请输入正确的时间跨度, 详情请查看changeDate()方法');
  } else {
    let span = parseInt(dateSpan.match(/-?\d+/g)[0] || 0, 10);
    span = span > 0 ? span : 0 - span;
    const spanText = changeDateToText(dateSpan);
    let ifValue = '';
    if (dateSpan.includes('M') || dateSpan.includes('y')) {
      ifValue =
        span > 0
          ? dateUtil.daysBetween(yesterdayFun(value[1]) || yesterdayFun(value), value[0] || new Date(), spanText.value) > span - 1
          : dateUtil.daysBetween(yesterdayFun(value[1]) || yesterdayFun(new Date()), value[0] || yesterdayFun(value), spanText.value) > span - 1;
    } else {
      ifValue =
        span > 0
          ? dateUtil.daysBetween(value[1] || value, value[0] || new Date(), spanText.value) > span - 1
          : dateUtil.daysBetween(value[1] || new Date(), value[0] || value, spanText.value) > span - 1;
    }
    const callbackText = `单次查询的时间跨度不得超过${span}个${spanText.text}`;
    dateFun(ifValue, dateSpanFix, spanText, callback, span, callbackText);
  }
};
const dateRangeFun = (dateRange, dateRangeFix, callback, value) => {
  if (!/^(-?\d+)([yMwdhms]s?)$/.test(dateRange)) {
    console.error('请输入正确的可操作时间长度, 详情请查看changeDate()方法');
  } else {
    const range = parseInt(dateRange.match(/-?\d+/g)[0] || 0, 10);
    const rangeText = changeDateToText(dateRange);
    const ifValue =
      range < 0
        ? dateUtil.daysBetween(new Date(), value[0], rangeText.value) > 0 - range - 1
        : dateUtil.daysBetween(value[1], new Date(), rangeText.value) > range - 1;
    const callbackText = `系统可提供近${range}${rangeText.text}内的结果查询`;
    dateFun(ifValue, dateRangeFix, rangeText, callback, range < 0 ? 0 - range : range, callbackText);
  }
};
const dateFun = (ifValue, dateRange, text, callback, span, callbackText) => {
  if (ifValue) {
    return dateRange ? callback(new Error(dateRange)) : callback(new Error(callbackText));
  }
};
export default {
  // 金额弱校验方法
  weakMoneyRule: function(rule, value, callback) {
    const maxLength = 12;
    if (typeof value === 'object') {
      const { min = 0, max = 0 } = value;
      if (typeof value.max === 'string') {
        const len0 = value.max.split('.')[0].length;
        if (len0 > maxLength) {
          return callback(new Error('最大查询金额为999 999 999 999, 请在0-999 999 999 999范围内查询'));
        }
      }
      if (typeof value.min === 'string') {
        const len1 = value.min.split('.')[0].length;
        if (len1 > maxLength) {
          return callback(new Error('最大查询金额为999 999 999 999, 请在0-999 999 999 999范围内查询'));
        }
      }
      if (parseFloat(min) > parseFloat(max)) {
        return callback(new Error('终止金额应大于起始金额'));
      }
      return callback();
    }
  },
  // 金额校验方法
  moneyRule: function(rule, value, callback) {
    const prefix = rule.prefix || '金额';
    const minAmount = rule.minAmount || 0;
    const anotherNumber = rule.anotherNumber && rule.anotherNumber(); //第二个金额
    const anotherNumberFix = rule.anotherNumberFix; //第二个金额-不符合的提示文字
    if (value !== '') {
      if (value === '.') {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      }
      if (Number(value) === 0) {
        return callback(new Error(`${prefix}不能为0`));
      }
      if (Number(minAmount) < 0) {
        console.error('请确认最低输入金额大于等于0', minAmount);
        return;
      }
      if (parseFloat(value) < parseFloat(minAmount)) {
        return callback(new Error(`${prefix}不可小于${minAmount}元`));
      }
      if (parseFloat(value) > parseFloat(anotherNumber)) {
        return callback(new Error(`${prefix}不能大于${anotherNumberFix}`));
      }
      return callback();
    }
  },
  // 最小金额不能小于等于0，或者不能小于其他自定义值
  minMoneyRule: function(rule, value, callback) {
    const prefix = rule.prefix || '金额';
    const minAmount = rule.minAmount || 0;
    const limit = rule.limit && rule.limit();
    if (value !== '') {
      if (value === '.') {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      }
      if (value > limit) {
        return callback(new Error(`${prefix}`));
      }
      if (Number(value) <= Number(minAmount)) {
        return callback(new Error(`${prefix}不可为${minAmount}元`));
      }
      return callback();
    }
  },
  // 校验数字类型
  checkCountRule: function(rule, value, callback) {
    const prefix = rule.prefix || '此处';
    const regu = /^[1-9]\d*$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}只能输入数字`));
      } else {
        return callback();
      }
    }
  },
  checkAcNameRule: function(rule, value, callback) {
    const prefix = rule.prefix || '用户名';
    const regu = /[`~!#$%^&*()_\\+-=<>?:"{}|~！#￥%……&*（）={}|《》？：“”【】、；‘’，。、\s+]/;
    if (value !== '') {
      if (regu.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  // 利率98
  rateCountRule: function(rule, value, callback) {
    const prefix = rule.prefix || '利率';
    const regu = /^[+]{0,1}(\d+)$|^[+]{0,1}(\d+\.\d+)$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  // 校验手机号
  phoneRule: function(rule, value, callback) {
    const prefix = rule.prefix || '手机号码';
    const regu = /^(13[0-9]|14[579]|15[0-3,5-9]|16[6]|17[0135678]|18[0-9]|19[89])\d{8}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  // 校验座机
  landlineRule: function(rule, value, callback) {
    const prefix = rule.prefix || '座机号码';
    const regu = /\d{3}-\d{8}|\d{4}-\d{7}/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  dateRangeRule: function(rule, value, callback) {
    const dateSpan = rule.dateSpan || null; // 时间跨度
    const dateSpanFix = rule.dateSpanFix; // 时间跨度 - 不符合的提示文字
    const dateRange = rule.dateRange || null; // 可以操作的最长时间
    const dateRangeFix = rule.dateRangeFix; // 可以操作的最长时间 - 不符合的提示文字
    if (value) {
      dateSpan && dateSpanFun(dateSpan, dateSpanFix, callback, value);
      dateRange && dateRangeFun(dateRange, dateRangeFix, callback, value);
    }
    return callback();
  },

  /**
   * @description 金额范围检测
   * @param {*} rule
   * @param {*} value
   * @param {*} callback
   */
  moneyRangeRule: function(rule, value, callback) {
    const prefix = rule.prefix || '金额';
    const min = value.min;
    const max = value.max;
    if (min && !max) {
      return callback(new Error(`终止${prefix}不能为空`));
    }
    if (!min && max) {
      return callback(new Error(`起始${prefix}不能为空`));
    }
    if (parseFloat(min) < 0 || parseFloat(max) > 99999999) {
      return callback(new Error(`您输入的${prefix}超出范围`));
    }
    if (parseFloat(max) < parseFloat(min)) {
      return callback(new Error(`您输入的${prefix}范围有误`));
    }
    return callback();
  },

  // 校验身份证号
  idCardRule: function(rule, value, callback) {
    const prefix = rule.prefix || '身份证号码';
    const regu = /(^\d{8}(0\d|10|11|12)([0-2]\d|30|31)\d{3}$)|(^\d{6}(18|19|20)\d{2}(0[1-9]|10|11|12)([0-2]\d|30|31)\d{3}(\d|X|x)$)/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`您输入的${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },

  // 校验email
  emailRule: function(rule, value, callback) {
    const prefix = rule.prefix || 'e-mail';
    const reg = /[\w!#$%&'*+/=?^_`{|}~-]+(?:\.[\w!#$%&'*+/=?^_`{|}~-]+)*@(?:[\w](?:[\w-]*[\w])?\.)+[\w](?:[\w-]*[\w])?/;
    if (value !== '') {
      if (reg.test(value)) {
        callback();
      } else {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      }
    }
  },
  // 校验账号
  accountRule: function(rule, value, callback) {
    const prefix = rule.prefix || '账户';
    const regu = /^\d{16,}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  // 校验6位数字
  verCodeRule: function(rule, value, callback) {
    const prefix = rule.prefix || '验证码';
    const num = rule.num || 6;
    const regu = eval('/^\\d{' + num + '}$/');
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    } else {
      return callback(new Error(`请输入${num}位数字${prefix}`));
    }
  },
  // 校验姓名
  nameRule: function(rule, value, callback) {
    const prefix = rule.prefix || '姓名';
    const regu = /^[\u4E00-\u9FA5]{2,6}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`您输入的${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  // 校验金额不低于1000.00元，并为1000的倍数
  amountRule: function(rule, value, callback) {
    const prefix = rule.prefix || '借款金额';
    const regu = /^(0|-?[1-9]\d*0{3}(.00)*)$/;
    if (!value) {
      return callback(new Error(`${prefix}不能为空！`));
    }
    if (value === '0') {
      return callback(new Error(`${prefix}不能为0！`));
    }
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  // 校验邮政编码
  postCodeRule: function(rule, value, callback) {
    const prefix = rule.prefix || '邮政编码';
    const regu = /^[0-9]{6}$/;
    if (value !== '') {
      if (!regu.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  // 校验只能输入英文/拼音
  onlyPYRule: function(rule, value, callback) {
    const prefix = rule.prefix || '英文/拼音';
    if (value !== '') {
      if (/[^a-zA-Z]/.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  //校验地址不能输入特殊字符
  onlyHZRule: function(rule, value, callback) {
    const prefix = rule.prefix || '地址';
    if (value !== '') {
      if (/[`!@#$%^&*()_+,.<>?:"{}|~！@#￥%……&*]/g.test(value)) {
        return callback(new Error(`${prefix}格式有误，请重新输入！`));
      } else {
        return callback();
      }
    }
  },
  // 处理密码控件form提交不做校验，通过自身方法校验
  pwdPass: function(rule, value, callback) {
    return callback();
  },
  // 处理ukeyform提交不做校验，通过自身方法校验
  ukeyRule: function(rule, value, callback) {
    return callback();
  },
  //还款本金不能大于贷款金额
  amountComparisonRule: function(rule, value, callback) {
    const moneyAFix = rule.moneyAFix; // 还款本金 - 不符合的提示文字
    const moneyB = rule.moneyB();
    const moneyBFix = rule.moneyBFix; // 贷款金额 - 不符合的提示文字
    if (value !== '') {
      if (value === '.') {
        return callback(new Error(`${moneyAFix}格式有误，请重新输入！`));
      }
      if (Number(value) < 0) {
        callback(new Error(`请确认最低输入金额大于等于0`));
      }
      if (parseFloat(value) > parseFloat(moneyB)) {
        return callback(new Error(`${moneyAFix}不能大于${moneyBFix}`));
      }
      return callback();
    }
  },
};
